#ifndef PVSQ_H
#define PVSQ_H

#define offsetof(_type, _field) ((int)(&((_type *)0)->_field))
#define fieldtotype(_fieldaddr, _typename, _fieldname) \
		(_typename *)(((char *)_fieldaddr) - offsetof(_typename, _fieldname))
typedef struct gen_queue {
	struct gen_queue *next;
	struct gen_queue *prev;
} gen_queue_t;

#define GENQ_INIT(q) \
		((q)->next = (q)->prev = (q))

#define GENQ_APPEND(q, msg) \
		((msg)->next = (q), \
		(msg)->prev = (q)->prev, \
		(q)->prev->next = (msg), \
		(q)->prev = (msg))

#define GENQ_PREPEND(q, msg) \
		((msg)->next = (q)->next, \
		(msg)->prev = (q), \
		(q)->next->prev = (msg), \
		(q)->next = (msg))

#define GENQ_DELETE(msg) \
		((msg)->next->prev = (msg)->prev, \
		(msg)->prev->next = (msg)->next, \
		(msg)->next = NULL, \
		(msg)->prev = NULL)

#define GENQ_NEXT(q) \
		((q)->next)

#define GENQ_PREV(q) \
		((q)->prev)

#define GENQ_FIRST(q) \
		((q)->next)

#define GENQ_LAST(q) \
		((q)->prev)

#define GENQ_EMPTY(q) \
		((q)->next == (q))
#define GENQ_FROM(q, type, field)		fieldtotype(q, type, field)

#define PVSQ_MAX_DEPTH 10

typedef struct {
  char    q_cmove[SANSZ];
  int     q_mdepth;
} pvs_entry_t;

typedef struct {
	gen_queue_t  pvs_ent_list;
	pvs_entry_t   pvs_ent;
} pvs_entry_list_t;


pvs_entry_list_t * pvsq_append(char *cmove, int mdepth);
void pvsq_cleanup(void);
boolean_t pvsq_find_ent(char *cmove, int mdepth);
void pvsq_update_stats(void);
void pvsq_update_stats_depth(int depth);
void pvsq_pr_stats(void);

#endif /* !PVSQ_H */
