#include <stdio.h>

#define offsetof(_type, _field) ((int)(&((_type *)0)->_field))
#define fieldtotype(_fieldaddr, _typename, _fieldname) \
		(_typename *)(((char *)_fieldaddr) - offsetof(_typename, _fieldname))
typedef struct gen_queue {
	struct gen_queue *next;
	struct gen_queue *prev;
} gen_queue_t;

#define GENQ_INIT(q) \
		((q)->next = (q)->prev = (q))

#define GENQ_APPEND(q, msg) \
		((msg)->next = (q), \
		(msg)->prev = (q)->prev, \
		(q)->prev->next = (msg), \
		(q)->prev = (msg))

#define GENQ_PREPEND(q, msg) \
		((msg)->next = (q)->next, \
		(msg)->prev = (q), \
		(q)->next->prev = (msg), \
		(q)->next = (msg))

#define GENQ_DELETE(msg) \
		((msg)->next->prev = (msg)->prev, \
		(msg)->prev->next = (msg)->next, \
		(msg)->next = NULL, \
		(msg)->prev = NULL)

#define GENQ_NEXT(q) \
		((q)->next)

#define GENQ_PREV(q) \
		((q)->prev)

#define GENQ_FIRST(q) \
		((q)->next)

#define GENQ_LAST(q) \
		((q)->prev)

#define GENQ_EMPTY(q) \
		((q)->next == (q))
#define GENQ_FROM(q, type, field)		fieldtotype(q, type, field)

typedef enum { FALSE, TRUE } boolean_t;


typedef struct {
  char    q_cmove[SANSZ];
  int     q_mdepth;
} pvs_entry_t;

typedef struct {
	gen_queue_t  pvs_ent_list;
	pvs_entry_t   pvs_ent;
} pvs_entry_list_t;
static gen_queue_t    pvsq;

static pvs_entry_list_t * pvsq_append(char *cmove, int mdepth);
static void pvsq_cleanup(void);
static boolean_t pvsq_find_ent(char *cmove, int mdepth);

main() 
{
	int i;
	GENQ_INIT(&pvsq);
	for (i = 0; i < 10; i++) {
		pvsq_append("hello", i);
	}

	for (i = 0; i < 11; i++) {
		if (pvsq_find_ent("hello", i) == FALSE) {
			pvsq_append("hello", i);
		}
	}

	for (i = 0; i < 11; i++) {
		pvsq_find_ent("hello", i);
	}
	pvsq_cleanup();
	return 0;
}

static pvs_entry_list_t *
pvsq_append(char *cmove, int mdepth)
{
	pvs_entry_list_t *pvs_listp;
	pvs_entry_t      *pvsp;

	pvs_listp = (pvs_entry_list_t *) malloc(sizeof(pvs_entry_list_t));
	if (pvs_listp == NULL) {
		printf("No memory\n");
		return NULL;
	}
	pvsp = &(pvs_listp->pvs_ent);
	bzero(pvsp, sizeof(pvs_entry_t));

	strcpy(pvsp->q_cmove, cmove);
	pvsp->q_mdepth  = mdepth;
	GENQ_APPEND(&pvsq, &(pvs_listp->pvs_ent_list));
	return pvs_listp;
}

static void
pvsq_cleanup(void)
{
	int i;
	
	while (!GENQ_EMPTY(&pvsq)) {
		gen_queue_t    *qp;
		pvs_entry_list_t *pvs_listp;
		qp        = GENQ_FIRST(&pvsq);
		pvs_listp = GENQ_FROM(qp, pvs_entry_list_t, pvs_ent_list);
		GENQ_DELETE(qp);
		free(pvs_listp);
	}
	return;
}

static boolean_t 
pvsq_find_ent(char *cmove, int mdepth)
{
	gen_queue_t     *qp;

	for (qp = GENQ_FIRST(&pvsq); qp != &pvsq; qp = GENQ_NEXT(qp)) {
		pvs_entry_list_t *pvs_listp;
		pvs_entry_t      *pvsp;
		pvs_listp = GENQ_FROM(qp, pvs_entry_list_t, pvs_ent_list);
		pvsp = &(pvs_listp->pvs_ent);
		if ((strcmp(pvsp->q_cmove, cmove) == 0) &&
			(pvsp->q_mdepth == mdepth)) {
			printf("FOUND. %s:%d\n", cmove, mdepth);
			return TRUE;
		}
	}
	return FALSE;
}
